#include	<stdio.h>
#include	<stdlib.h>

#include	<X11/X.h>
#include	<X11/Xos.h>
#include	<X11/Xlib.h>
#include	<X11/Xutil.h>

#include	"wm.h"

int current_item;		/* Last known selected menu item. -1 if none. */
int current_items;	/* Number of items on the menu when popped up. */

static Client *hiddenc[MAXHIDDEN];
static int numhidden;
static char *menuitems[1 + MAXHIDDEN + 1] = {
	"New Shell", 0
};


void
getmouse(int *x, int *y)
{
	Window dw1, dw2;
	int t1, t2;
	unsigned int b;

	XQueryPointer(dpy, root, &dw1, &dw2, x, y, &t1, &t2, &b);
}

int
menu_whichitem(int x, int y)
{
	int width;	/* Width of menu. */
	int height;	/* Height of each menu item. */

	menu_dimensions(&width, &height);

	/*
	 * Translate to popup window coordinates. We do this ourselves to avoid
	 * a round trip to the server.
	 */
	x -= start_x;
	y -= start_y;

	/*
	 * Are we outside the menu?
	 */
	if (x < 0 || x > width || y < 0 || y >= current_items * height)
		return -1;

	return y / height;
}

void
menu_dimensions(int *width, int *height)
{
	int w;	/* Widest string so far. */
	int i;	/* Menu item. */

	w = 0;
	for (i = 0; menuitems[i]; i++) {
		int tw = XTextWidth(popup_font, menuitems[i], strlen(menuitems[i])) + 4;
		if (tw > w) w = tw;
	}

	*width = w + border;
	*height = popup_font->ascent + popup_font->descent + 1;
}

void
menuhit(XButtonEvent *e)
{
	int width;	/* Width of menu. */
	int height;	/* Height of each menu item. */

	menu_dimensions(&width, &height);

	/*
	 * Arrange for centre of "New Shell" menu item to be under pointer.
	 */
	start_x = e->x - width / 2;
	start_y = e->y - height / 2;

	current_item = 0;
	current_items = numhidden + 1;

	XMoveResizeWindow(dpy, popup, start_x, start_y,
		width, current_items * height);
	XMapRaised(dpy, popup);
	XChangeActivePointerGrab(dpy, ButtonMask | ButtonMotionMask |
		OwnerGrabButtonMask, None, CurrentTime);

	mode = wm_menu_up;
}

void 
hide(Client * c)
{
	if (c == 0 || numhidden == MAXHIDDEN)
		return;

	XUnmapWindow(dpy, c->window);
	XUnmapWindow(dpy, c->parent);

	/*
	 *	If the window was the current window, it isn't any more...
	 */
	setactive(c, 0);
	Client_SetState(c, IconicState);
	if (current == c)
		current = 0;

	hiddenc[numhidden] = c;
	menuitems[1 + numhidden] = (c->name != 0) ? c->name : "<unknown>";
	numhidden++;
	menuitems[1 + numhidden] = 0;
}

void
unhide(int n, int map)
{
	Client	*c;
	int	i;

	if (n < 0 || n > numhidden) return;

	c = hiddenc[n];

	if (map) {
		XMapWindow(dpy, c->window);
		XMapRaised(dpy, c->parent);
		Client_SetState(c, NormalState);
	}
	numhidden--;
	for (i = n; i < numhidden; i++) {
		hiddenc[i] = hiddenc[i + 1];
		menuitems[1 + i] = menuitems[1 + i + 1];
	}
	menuitems[1 + numhidden] = 0;
}

void
unhidec(Client *c, int map)
{
	int	i;

	for (i = 0; i < numhidden; i++) {
		if (c == hiddenc[i]) {
			unhide(i, map);
			return;
		}
	}
}

void
update_menu(Client *c)
{
	int	i;

	if (c == 0)
		return;

	for (i = 0; i < numhidden; i++) {
		if (c == hiddenc[i]) {
			menuitems[1 + i] = (c->name != 0) ? c->name : "<unknown>";
			return;
		}
	}
}

void
menu_expose(void)
{
	int i;		/* Menu item being drawn. */
	int width;	/* Width of each item. */
	int height;	/* Height of each item. */

	menu_dimensions(&width, &height);

	/*
	 * Redraw the labels.
	 */
	for (i = 0; i < current_items; i++) {
		int tx = (width - XTextWidth(popup_font, menuitems[i],
			strlen(menuitems[i]))) / 2;
		int ty = i * height + popup_font->ascent + 1;

		XDrawString(dpy, popup, popup_gc, tx, ty, menuitems[i],
			strlen(menuitems[i]));
	}

	/*
	 * Highlight current item if there is one.
	 */
	if (current_item >= 0 && current_item < current_items)
		XFillRectangle(dpy, popup, popup_gc, 0, current_item * height,
			width, height);
}

void
menu_motionnotify(XEvent* ev)
{
	int x, y;		/* Event position. */
	int old;			/* Old menu position. */
	int width;		/* Width of menu. */
	int height;		/* Height of each menu item. */
	XButtonEvent *e = &ev->xbutton;

	x = e->x;
	y = e->y;

	menu_dimensions(&width, &height);

	old = current_item;
	current_item = menu_whichitem(x, y);

	if (current_item == old) return;

	/* 
	 * Unhighlight the old position, if it was on the menu.
	 */
	if (old >= 0 && old < current_items)
		XFillRectangle(dpy, popup, popup_gc, 0, old * height, width, height);

	/*
	 * Highlight the new position, if it's on the menu.
	 */
	if (current_item >= 0 && current_item < current_items)
		XFillRectangle(dpy, popup, popup_gc, 0, current_item * height,
			width, height);
}
